<?php

// for generate transaction

require_once "Signature.php";

class ipay88_atm_transfer_payment_Transaction
{
    protected $merchantKey;
    protected $merchantCode;
    protected $paymentId;
    protected $refNo;
    protected $amount;
    protected $currency;
    protected $status;
    protected $transactionId;
    protected $signature;
    protected $data;

    // @return mixed
    public function getData()
    {
        return $this->data;
    }

    // @param mixed $data
    public function setData($data)
    {
        $this->data = $data;
    }

    // @return mixed
    public function getTransactionId()
    {
        return $this->transactionId;
    }

    // @param mixed $transactionId
    public function setTransactionId($transactionId)
    {
        $this->transactionId = $transactionId;
    }

    // @return mixed
    public function getSignature()
    {
        return $this->signature;
    }

    // @param mixed $signature
    public function setSignature($signature)
    {
        $this->signature = $signature;
    }

    public function __construct($options = array())
    {
        $options = $this->formartDataOption($options);

        if (count($options)) {
            foreach ($options as $key => $value) {
                $this->$key = $value;
            }
        }
    }

    public function formartDataOption($options)
    {
        return array(
            'merchantCode'  => isset($options['MerchantCode']) ? $options['MerchantCode'] : '',
            'refNo'         => isset($options['RefNo']) ? $options['RefNo'] : '',
            'amount'        => isset($options['Amount']) ? $options['Amount'] : '',
            'currency'      => isset($options['Currency']) ? $options['Currency'] : '',
            'status'        => isset($options['Status']) ? $options['Status'] : '',
            'transactionId' => isset($options['TransId']) ? $options['TransId'] : '',
            'paymentId'     => isset($options['PaymentId']) ? $options['PaymentId'] : '',
        );
    }

    public function isPaid($signature = null)
    {
        $signature = $signature ?: $this->getSignature();

        if ($this->checkSignature($signature) && $this->getStatus() == ipay88_atm_transfer_payment_Config::PAYMENT_STATUS_SUCCESS) {
            return true;
        }

        return false;
    }

    public function isWaitingPayment($signature = null)
    {
        $signature = $signature ?: $this->getSignature();

        if ($this->checkSignature($signature) && $this->getStatus() == ipay88_atm_transfer_payment_Config::PAYMENT_STATUS_PENDING) {
            return true;
        }

        return false;
    }

    private function checkSignature($signature)
    {
        $expectedSignature = $this->generateSignature();

        if ($signature == $expectedSignature) {
            return true;
        }else{
			WC_iPay88_atm::add_debug_log("checkSignature $signature different than expected $expectedSignature");
		}

        return false;
    }

    private function generateSignature()
    {
        $merchantKey    = $this->getMerchantKey();
        $merchantCode   = $this->getMerchantCode();
        $paymentId      = $this->getPaymentId();
        $refNo          = $this->getRefNo();
        $amount         = $this->getAmount();
        $currency       = $this->getCurrency();
        $status         = $this->getStatus();

        $source = $merchantKey . $merchantCode . $paymentId . $refNo . $amount . $currency . $status;

        $signature = new ipay88_atm_transfer_payment_Signature($source);

        return $signature->getSignature();
    }

    // @return mixed
    public function getMerchantKey()
    {
        return $this->merchantKey;
    }

    // @param mixed $merchantKey
    public function setMerchantKey($merchantKey)
    {
        $this->merchantKey = $merchantKey;
    }

    // @return mixed
    public function getMerchantCode()
    {
        return $this->merchantCode;
    }

    // @param mixed $merchantCode
    public function setMerchantCode($merchantCode)
    {
        $this->merchantCode = $merchantCode;
    }

    // @return mixed
    public function getPaymentId()
    {
        return $this->paymentId;
    }

    // @param mixed $paymentId
    public function setPaymentId($paymentId)
    {
        $this->paymentId = $paymentId;
    }

    // @return mixed
    public function getRefNo()
    {
        return $this->refNo;
    }

    // @param mixed $refNo
    public function setRefNo($refNo)
    {
        $this->refNo = $refNo;
    }

    // @return mixed
    public function getAmount()
    {
        return $this->amount;
    }

    // @param mixed $amount
    public function setAmount($amount)
    {
        $this->amount = $amount;
    }

    // @return mixed
    public function getCurrency()
    {
        return $this->currency;
    }

    // @param mixed $currency
    public function setCurrency($currency)
    {
        $this->currency = $currency;
    }

    // @return mixed
    public function getStatus()
    {
        return $this->status;
    }

    // @param mixed $status
    public function setStatus($status)
    {
        $this->status = $status;
    }
}
